import sys
import typing
import allure
from qrunner.utils.log import logger
from qrunner.utils.config import conf
from qrunner.core.android.driver import AndroidDriver
from qrunner.core.android.element import AndroidElement
from qrunner.core.ios.driver import IosDriver
from qrunner.core.ios.element import IosElement
from qrunner.core.browser.driver import BrowserDriver
from qrunner.core.browser.element import WebElement
from qrunner.core.api.request import HttpRequest, ResponseResult
import jmespath
# from jsonschema import validate
# from jsonschema.exceptions import ValidationError


class TestCase(HttpRequest):
    """
    测试用例基类，所有测试用例需要继承该类
    """
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        # 初始化driver
        logger.info('初始化driver')
        # 从配置文件中获取浏览器相关配置（为了支持并发执行）
        platform = conf.get_name('common', 'platform')
        serial_no = conf.get_name('app', 'serial_no')
        browser_name = conf.get_name('web', 'browser_name')

        cls.driver: typing.Union[AndroidDriver, IosDriver, BrowserDriver] = None
        if platform == 'android':
            if serial_no:
                cls.driver = AndroidDriver(serial_no)
            else:
                logger.info('serial_no为空')
                sys.exit()
        elif platform == 'ios':
            if serial_no:
                cls.driver = IosDriver(serial_no)
            else:
                logger.info('serial_no为空')
                sys.exit()
        elif platform == 'browser':
            cls.driver = BrowserDriver(browser_name)
        elif platform == 'api':
            pass
        else:
            logger.info(f'不支持的平台: {platform}')
            sys.exit()
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        logger.info('teardown_class')
        platform = conf.get_name('common', 'platform')
        logger.info(platform)
        if platform == 'browser':
            cls().driver.quit()
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.platform = conf.get_name('common', 'platform')
        if self.platform in ['android', 'ios']:
            self.driver.force_start_app()
        self.start()

    def teardown_method(self):
        self.end()
        if self.platform in ['android', 'ios']:
            self.driver.stop_app()
        # if self.platform == 'api':
        #     method = ResponseResult.method
        #     path: str = ResponseResult.path
        #     allure.dynamic.title(f'[{method}]{path}')  # 设置用例标题

    # @staticmethod
    # def set_title(text):
    #     """
    #     设置allure报表中对应用例的标题
    #     @param text: 测试用例标题
    #     @return:
    #     """
    #     allure.dynamic.title(text)

    def el(self, **kwargs):
        """
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        if self.platform == 'android':
            element = AndroidElement(**kwargs)
        elif self.platform == 'ios':
            element = IosElement(**kwargs)
        elif self.platform == 'browser':
            element = WebElement(self.driver, **kwargs)
        else:
            logger.info(f'不支持的平台: {self.platform}，暂时只支持android、ios、browser')
            sys.exit()
        return element

    def click(self, **kwargs):
        self.el(**kwargs).click()

    def assertExist(self, **kwargs):
        assert self.el(**kwargs).exists(timeout=3)

    def open(self, url, cookies: list = None):
        """
        访问链接为url的页码
        :param url: 页面链接
        :param cookies: 登录相关的cookie，如[
            {'name': 'xxx', 'value': 'xxxx'}
        ]
        :return:
        """
        self.driver.open_url(url)
        if cookies:
            self.driver.add_cookies(cookies)
            self.driver.refresh()

    def screenshot(self, file_name: str):
        """
        截图并存为文件
        :param file_name: 如test.png
        :return:
        """
        if not file_name.endswith('.png'):
            file_name = file_name + '.png'
        self.driver.screenshot(file_name)

    def assertStatusCode(self, status_code):
        """
        Asserts the HTTP status code
        """
        assert ResponseResult.status_code == status_code, \
            f'status_code {ResponseResult} != {status_code}'

    # def assertSchema(self, schema):
    #     """
    #     Assert JSON Schema
    #     doc: https://json-schema.org/
    #     """
    #     try:
    #         validate(instance=ResponseResult.response, schema=schema)
    #     except ValidationError as msg:
    #         assert "Response data" == "Schema data", msg
    #     else:
    #         assert True

    # def assertJSON(self, assert_json):
    #     """
    #     Assert JSON data
    #     """
    #     AssertInfo.data = []
    #     diff_json(ResponseResult.response, assert_json)
    #     if len(AssertInfo.data) == 0:
    #         self.assertTrue(True)
    #     else:
    #         self.assertEqual("Response data", "Assert data", msg=AssertInfo.data)

    def assertPath(self, path, value):
        """
        Assert path data
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value == value, f'{search_value} != {value}'

    def assertLocalPath(self, path, value, json_obj):
        """
        同assertPath，只是源数据不是请求响应，而是传入的json_ojb
        @param path:
        @param value:
        @param json_obj:
        @return:
        """
        search_value = jmespath.search(path, json_obj)
        assert search_value == value, f'{search_value} != {value}'

    def assertLenEq(self, path, value):
        """
        断言列表长度等于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert len(search_value) == value, f"{search_value} 的长度不等于 {value}"

    def assertLenGt(self, path, value):
        """
        断言列表长度大于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert len(search_value) > value, f"{search_value} 的长度不大于 {value}"

    def assertGt(self, path, value):
        """
        值大于多少
        :param path:
        :param value:
        :return:
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value > value, f"{search_value} 不大于 {value}"

    def assertGe(self, path, value):
        """
        值大于等于
        :param path:
        :param value:
        :return:
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value >= value, f"{search_value} 小于 {value}"

    def assertNotEq(self, path, value):
        """值不等于"""
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value != value, f"{search_value} 等于 {value}"

    def assertIn(self, path, value_list: list):
        """
        断言匹配结果被value_list包含
        @param path:
        @param value_list:
        @return:
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value in value_list, f"{value_list} 不包含 {search_value}"

    def assertContains(self, path, value):
        """
        断言匹配结果包含value
        @param path:
        @param value:
        @return:
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert value in search_value, f"{search_value} 不包含 {value}"

    def assertType(self, path, value_type):
        if not isinstance(value_type, type):
            if value_type == 'int':
                value_type = int
            elif value_type == 'str':
                value_type = str
            elif value_type == 'list':
                value_type = list
            elif value_type == 'dict':
                value_type = dict
            else:
                value_type = str

        search_value = jmespath.search(path, ResponseResult.response)
        assert isinstance(search_value, value_type), f'{search_value} 不是 {value_type} 类型'

    def assertCode(self, code):
        """
        断言返回json中的code
        @param code: 0
        @return:
        """
        self.assertPath('code', code)


class Page:
    """
    测试页面基类，所有页面需要继承该类
    """
    def __init__(self, driver, url=None):
        """
        :param driver: 驱动句柄
        :param url: 页面链接
        :param cookies: 登录态相关cookies: [
            {'name': 'xxx', 'value': 'xxxx'},
            {'name': 'xxx', 'value': 'xxxx'},
        ]
        """
        self.platform = conf.get_name('common', 'platform')
        self.driver = driver
        if url is not None:
            self.url = url
            self.open(self.url)

    def el(self, **kwargs):
        """
        :param args: 暂时无用
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        element: typing.Union[AndroidElement, IosElement, WebElement] = None
        if self.platform == 'android':
            element = AndroidElement(**kwargs)
        elif self.platform == 'ios':
            element = IosElement(**kwargs)
        elif self.platform == 'browser':
            element = WebElement(self.driver, **kwargs)
        else:
            logger.info(f'不支持的平台: {self.platform}，暂时只支持android、ios、browser')
            sys.exit()
        return element

    def open(self, url, cookies: str = None):
        """
        访问链接为url的页码
        :param url: 页面链接
        :@param cookies: 登录相关的cookie，如[
            {'name': 'xxx', 'value': 'xxx'}
        ]
        :return:
        """
        self.driver.open_url(url)
        if cookies:
            self.driver.add_cookies(cookies)
            self.driver.refresh()

    def screenshot(self, file_name):
        """
        截图并存为文件
        :param file_name: 如test.png
        :return:
        """
        if not file_name.endswith('.png'):
            file_name = file_name + '.png'
        self.driver.screenshot(file_name)

    # def allure_shot(self, file_name):
    #     """
    #     截图并上传至allure
    #     :param file_name: 如首页截图
    #     :return:
    #     """
    #     self.driver.upload_pic(file_name)


class H5Page:
    """
    测试页面基类，所有页面需要继承该类
    """
    def __init__(self, driver):
        """
        :param driver: 驱动句柄
        :param url: 页面链接
        :param cookies: 登录态相关cookies: [
            {'name': 'xxx', 'value': 'xxxx'},
            {'name': 'xxx', 'value': 'xxxx'},
        ]
        """
        self.driver = driver

    def el(self, **kwargs):
        """
        :param args: 暂时无用
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        element: WebElement = WebElement(self.driver, **kwargs)
        return element

    def open(self, url, cookies: str = None):
        """
        访问链接为url的页码
        :param url: 页面链接
        :@param cookies: 登录相关的cookie，如[
            {'name': 'xxx', 'value': 'xxx'}
        ]
        :return:
        """
        self.driver.open_url(url)
        if cookies:
            self.driver.add_cookies(cookies)
            self.driver.refresh()

    def screenshot(self, file_name):
        """
        截图并存为文件
        :param file_name: 如test.png
        :return:
        """
        if not file_name.endswith('.png'):
            file_name = file_name + '.png'
        self.driver.screenshot(file_name)

    def allure_shot(self, file_name):
        """
        截图并上传至allure
        :param file_name: 如首页截图
        :return:
        """
        self.driver.upload_pic(file_name)



import qrunner


class TestApiSample(qrunner.TestCase):

    @qrunner.file_data('card_type', 'data.json')
    def test_case_01(self, card_type):
        path = '/api/qzd-bff-app/qzd/v1/home/getToolCardListForPc'
        pay_load = {"type": card_type}
        self.post(path, json=pay_load)



import time
from qrunner.utils.faker import Faker


class MockData:
    def __init__(self, locale=None):
        """
        指定语言
        @param locale: 默认英文en_US， 中文是zh_CN
        """
        if locale is not None:
            self.fake = Faker(locale)
        else:
            self.fake = Faker()

    @property
    def now_time(self):
        """
        当前时间
        @return: 年-月-日 时-分-秒
        """
        return time.strftime("%Y-%m-%d %H:%M:%S")

    @staticmethod
    def timestamp(n=10):
        """
        当前时间戳，10位是秒，13位是毫秒
        @param n: 指定时间戳的位数，不够位数右边补充6
        @return: 时间戳
        """
        _timestamp = (str(time.time()).replace('.', '')[:n]).ljust(n, '6')
        return _timestamp

    @property
    def date(self):
        """
        当前日期
        @return: 年-月-日
        """
        return self.fake.date(pattern="%Y-%m-%d")

    @property
    def timezone(self):
        """时区"""
        return self.fake.timezone()

    @property
    def name(self):
        """
        姓名
        @return: 设置的locale不同，返回的语言文本不同
        """
        return self.fake.name()

    @property
    def company(self):
        """
        公司名
        @return:
        """
        return self.fake.company()

    @property
    def address(self):
        """
        地址
        @return:
        """
        return self.fake.address()

    @property
    def province(self):
        """
        省
        @return:
        """
        return self.fake.province()

    @property
    def city(self):
        """
        市
        @return:
        """
        return self.fake.city()

    @property
    def county(self):
        """
        区
        @return:
        """
        return self.fake.district()

    @property
    def street(self):
        """
        街道
        @return:
        """
        return self.fake.street_address()

    @property
    def phone_number(self):
        # """
        # 根据时间戳生成一个不重复的手机号
        # @return: 13652435335
        # """
        # # 移动手机号前几位
        # cm = [134, 135, 136, 137, 138, 139, 150, 151, 152, 157, 158, 159, 182, 183, 184, 187, 188, 147, 178, 1705]
        # # 联通手机号前几位
        # cu = [130, 131, 132, 155, 156, 185, 186, 145, 176, 1709]
        # # 手机号前几位
        # ct = [133, 153, 180, 181, 189, 177, 1700]
        # phone = str(random.choice(cm + cu + ct)) + MockData.timestamp(7)
        # phone = phone[:11]
        return self.fake.phone_number()

    @property
    def credit_card_number(self):
        """
        信用卡号码
        @return:
        """
        return self.fake.credit_card_number()

    @property
    def email(self):
        """邮箱"""
        return self.fake.email()

    @property
    def ssn(self):
        """身份证"""
        return self.fake.ssn()

    @property
    def word(self):
        """关键词"""
        return self.fake.word()

    def words(self, n):
        """关键词列表"""
        return self.fake.words(nb=n)


mock = MockData('zh_CN')
mock_en = MockData()


if __name__ == '__main__':
    print(mock.name)
    print(mock_en.name)













# @Time    : 2022/2/22 10:37
# @Author  : kang.yang@qizhidao.com
# @File    : decorate.py
import json
import os

import allure
import pytest
import inspect as sys_inspect


def feature(text):
    return allure.feature(text)


def story(text):
    return allure.story(text)


# 兼容历史版本
def module(text):
    return allure.story(text)


def title(text):
    return allure.title(text)


def data(*args, **kwargs):
    return pytest.mark.parametrize(*args, **kwargs)


# 有空把文件装饰器补上
def file_data(data_key=None, file_name=None):
    # 获取被装饰方法的目录
    stack_t = sys_inspect.stack()
    ins = sys_inspect.getframeinfo(stack_t[1][0])
    file_dir = os.path.dirname(os.path.abspath(ins.filename))
    parent_dir = os.path.dirname(file_dir)
    parent_dir_dir = os.path.dirname(parent_dir)
    parent_dir_dir_dir = os.path.dirname(parent_dir_dir)

    # 判断当前、父目录、爷目录、太爷爷目录下的test_data目录下是否有file_name文件
    path_list = [file_dir, parent_dir, parent_dir_dir, parent_dir_dir_dir]
    file_path = None
    full_list = []
    for _path in path_list:
        full_path = os.path.join(_path, 'test_data', file_name)
        full_list.append(full_path)
        # print(full_path)
        if os.path.isfile(full_path) is True:
            file_path = full_path
            break
    if file_path is None:
        raise Exception(f"can not found {file_name} in {full_list}")

    # 根据文件路径以及key获取文件中的数据，暂时只支持json文件
    with open(file_path, 'r', encoding='utf-8') as f:
        json_data = json.load(f)

    content = json_data[data_key]
    return data(data_key, content)




